# -*- coding: utf-8 -*-

"""
Package tmpl
=======================================

Top-level package for tmpl.
"""

__version__ = "0.0.0"

import re,os
from pathlib import Path

def expand_string(s, parameters):
    for parameter,value in parameters.items():
        pattern = '{{tmpl.' + parameter +'}}'
        s = s.replace(pattern, value)
    validate(s)
    return s

def validate(s):
    """Verify that there are no more variables to be replaced.

    :param str s: string to be validated
    :raise: ValueError if not fully expanded.
    """
    pattern = r'{{tmpl\.(\w+)}}'
    m = re.search(pattern, s)
    if m:
        raise ValueError(f"Missing parameter: '{m[1]}'.")


def expand_file(root, path_to_template, destination, parameters):
    """Expand a single template file. Both path and contents are expanded.

    :param Path root: path to parent directory of the filepath to be expanded. The root is
        excluded from the expansion.
    :param Path path_to_template: location of of the template file
    :param Path destination: path to folder where the template file is to be expanded, with
        its relative path to root.
    :param dict parameters: dictionary with the variatbles and their corresponding values.
        All occurences of '{{tmpl.variable}}' in the template file and its filename are
        replaced with parameters[variable].
    """
    # Expand the file contents
    template = path_to_template.read_text()
    try:
        template = expand_string(template, parameters)
    except ValueError as exc:
        msg = exc.args[0] + f"\n    while expanding contents of file {path_to_template}."
        raise ValueError(msg)

    # expand the file path
    filepath = str(path_to_template.relative_to(root))
    try:
        filepath = expand_string(filepath, parameters)
    except ValueError as exc:
        msg = exc.args[0] + f"\n    while expanding path of file {path_to_template}."
        raise ValueError(msg)

    # Write the result
    dst = destination / filepath
    dst.parent.mkdir(parents=True, exist_ok=True)
    dst.write_text(template)


def expand_folder(path_to_template_folder, destination, parameters):
    """Expand a template folder.

    :param Path path_to_template_folder: location of the template folder. Only the contents
        of path_to_template_folder are expanded, not the folder itself.
    :param Path destination: path to folder where the template folder is to be expanded.
        The filename of the destination file is the filename of the template, after replacing
        the parameters
    :param dict parameters: dictionary with the variatbles and their corresponding values.
        All occurences of '{{tmpl.variable}}' in the template file and its filename are
        replaced with parameters[variable].
    """
    root = path_to_template_folder
    for d, dirs, files in os.walk(path_to_template_folder):
        for f in files:
            if f != '.DS_Store':
                try:
                    expand_file(root, Path(d) / f, destination, parameters)
                except ValueError as exc:
                    msg = exc.args[0] + f'\n    while expanding template folder {path_to_template_folder}.'
                    raise ValueError(msg)


# Script for installing the project in the current Python environment in editable mode

import sys
import subprocess
from pathlib import Path
import shutil
import os
import pkg_resources


def get_base_prefix_compat():
    """Get base/real prefix, or sys.prefix if there is none."""
    result = getattr(sys, "base_prefix", None) or getattr(sys, "real_prefix", None) or sys.prefix
    # print(f"get_base_prefix_compat() = {result}")
    return result


def in_virtualenv():
    return get_base_prefix_compat() != sys.prefix


def main():
    """Make an editable install of project `project` in the Python environment with which
    this script was called. Editable installs make changes to your project's source code
    instantly visible in the environment, so they can be tested and debugged.

    This script is a work around for ``pip install -e path/to/project`` which fails for
    micc projects because pip requires a ``setup.py`` file for editable installs.

    This script may be called with *any* Python executable, not necessarily the one of
    yoyr Micc2 environment. The project's package will bee installed in the site-packages
    of the calling Python. If the calling Python is not in a virtual environment the project
    is installed with the `--user` flag to avoid cluttering the system Python's site-packages.

    :param str project: path to the project to install, defaults to CWD.

    This script uses pip to install a package, and then replaces the package in the
    site-packages directory with a symlink to the source package in the project directory.
    """
    if len(sys.argv) > 1:
        project = sys.argv[1]
    else:
        project = '.'
    print(project)
    project_path = Path(project).resolve()
    print(project_path)

    if not project_path.exists():
        raise RuntimeError(f"Project path {project_path} does not exist.")

    if not project_path.is_dir():
        raise RuntimeError(f"Not a project directory: {project_path}.")

    if not (project_path / 'pyproject.toml').is_file():
        raise RuntimeError(f"Not a project directory: {project_path}.")

    project_name = project_path.name
    package_name = project_name.replace('-', '_').lower()

    package_path = (project_path / package_name)
    if package_path.is_dir():
        if not (package_path / '__init__.py').is_file():
            raise RuntimeError(f"Not a project directory: {project_path}. (Package {package_name} not found.)")
        else:
            structure = 'package'
    else:
        if not (project_path / f'{package_name}.py').is_file():
            raise RuntimeError(f"Not a project directory: {project_path}. (Module {package_name} not found.)")
        else:
            structure =' module'

    if in_virtualenv():
        user_flag = False
        environment = 'virtual environment'
    else:
        user_flag = True
        environment = 'current Python environment'
        # print(f'Install (editable) project {project_name} in current Python environment {sys.prefix} with `--user`?')

    print(f"Create editable install of project `{project_name}` in {environment} (={sys.prefix})?")
    while 1:
        answer = input('Proceed (yes/no)? ')
        if answer.lower().startswith('n'):
            print('Interrupted.')
            sys.exit(-1)
        if answer == '' or answer.lower().startswith('y'):
            break
    print("\nProceeding ...")
    # install, not editable
    cmd = [sys.executable, "-m", "pip", "install", str(project_path)]
    if user_flag:
        cmd.insert(4,'--user')
    print(f"> {' '.join(cmd)}")
    subprocess.check_call(cmd)
    try:
        pkg_dist_info = pkg_resources.get_distribution(package_name)
    except pkg_resources.DistributionNotFound:
        print(f'Package {package_name} not found.')
        raise
    else:
        location = pkg_dist_info.location
        print(f'Package `{package_name}` installed at `{location}`.\n')

    # make installation editable
    if structure == 'package':
        package_path = Path(location) / package_name
        print(f'Removing package: {package_path}')
        if package_path.is_symlink():
            package_path.unlink()
        else:
            shutil.rmtree(package_path)
        
        # Occasionally, we must remove a module file. If the project had a module structure, but
        # was recently converted to a package structure, an orphan module file (package_name.py)
        # may be left in the site-packages directory
        module = f'{package_name}.py'
        module_path = package_path / module
        if module_path.exists():
            module_path.unlink()
            
        print(f'Replacing package with symbolic link: {project_path / package_name}')
        os.symlink(src=(project_path / package_name), dst=package_path)

    else: # module structure
        module = f'{package_name}.py'
        package_path = Path(location) / module
        print(f'Removing package: {package_path}')
        package_path.unlink()
        print(f'Replacing package with symbolic link: {project_path / module}')
        os.symlink(src=(project_path / module), dst=package_path)

    print(f"\nEditable install of {project_name} is ready.\n")


if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-

"""
Package {{tmpl.package_name}}
=======================================

Top-level package for {{tmpl.package_name}}.
"""

__version__ = "{{tmpl.version}}"


def hello(who='world'):
    """'Hello world' method.

    :param str who: whom to say hello to
    :returns: a string
    """
    result = "Hello " + who
    return result

# Your code here...


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for C++ module {{tmpl.import_lib}}.
"""

import sys
sys.path.insert(0,'.')

import numpy as np

import {{tmpl.import_lib}}


def test_cpp_add():
    x = np.array([0,1,2,3,4],dtype=float)
    shape = x.shape
    y = np.ones (shape,dtype=float)
    z = np.zeros(shape,dtype=float)
    expected_z = x + y
    result = {{tmpl.import_lib}}.add(x,y,z)
    assert (z == expected_z).all()


#===============================================================================
# The code below is for debugging a particular test in eclipse/pydev.
# (normally all tests are run with pytest)
#===============================================================================
if __name__ == "__main__":
    the_test_you_want_to_debug = test_cpp_add

    print(f"__main__ running {the_test_you_want_to_debug} ...")
    the_test_you_want_to_debug()
    print('-*# finished #*-')
#===============================================================================



# -*- coding: utf-8 -*-
# By making tests a python package you can split off any helper functions in
# separate source files, which can then be imported wherever you need them.
"""
The tests package for {{tmpl.project_name}}.
"""



# -*- coding: utf-8 -*-
"""
module config
=============

"""
from pathlib import Path
import json
import click

class Config:
    """Class for configuration (or preferences) files.

    In fact little more than a persistent dict."""

    # this key controls the file location of the Config file
    file_key = 'file_loc'

    def __init__(self, **kwargs ):
        """"""
        self.data = {}

        if Config.file_key in kwargs:
            # Read Config object data
            p_cfg = Path(kwargs[Config.file_key])
            self.load(p_cfg)

        # Add the other kwargs
        kwargs.pop(Config.file_key,None)
        self.add(**kwargs)

    def load(self, p_cfg):
        """

        :param Path p_cfg: path to cfg file
        """
        if p_cfg.is_absolute():
            if p_cfg.exists():
                self.p_cfg = p_cfg
            else:
                raise FileNotFoundError(p_cfg)
        else:
            # first look in the current directory
            if p_cfg.exists():
                self.p_cfg = p_cfg.resolve()

            # then look in the home directory:
            elif (Path.home() / p_cfg.name).exists():
                self.p_cfg = Path.home() / p_cfg.name
            
            else:
                raise FileNotFoundError(p_cfg.name)

        with self.p_cfg.open() as fp_cfg:
            data = json.load(fp_cfg)
            self.data.update(data)
        self.update_location_(self.p_cfg)


    def save(self, file='', mkdir=False):
        """Save self.data to cfg file. If file is not specified """
        if file:
            p_cfg =  Path(file)
        elif Config.file_key in self.data:
            p_cfg = Path(self.data[Config.file_key])
        else:
            raise RuntimeError("Don't have a file location to save to.")

        p_cfg = self.p_cfg if not file else Path(file)
        p_cfg = p_cfg.resolve()
        if not p_cfg.parent.exists():
            if mkdir:
                p_cfg.parent.mkdir()
            else:
                print(f'Inexisting Path: {p_cfg.parent}. Create it yourself, or use the "mkdir=True" parameter.')
                raise FileNotFoundError()
        self.update_location_(p_cfg)

        with p_cfg.open('w') as fp_cfg:
            json.dump(self.data, fp_cfg, indent=2)


    def add(self, **kwargs):
        self.data.update(kwargs)


    # "Private" methods: For internal use only.
    def update_location_(self, p_cfg):
        """Make sure self.data[Config.file_key] points to the correct location, i.e. where the Config object was
        Read from or written to.

        For internal use only.
        """
        self.data[Config.file_key] = str(p_cfg)


    def __str__(self):
        return json.dumps(self.data,indent=2)


    def __getitem__(self, item):
        return self.data[item]


    def __setitem__(self, key, value):
        self.data[key] = value

    
    def update(self,dict):
        self.data.update(dict)

def get_answer():
    """Get answer str, raise KeyboardInterrupt if answer contains `^^`."""
    click.secho('Enter `^^` to exit.', fg='white')
    answer = input('>: ')
    if '^^' in answer:
        raise KeyboardInterrupt
    return answer


def get_param(name,description):
    """"""
    if 'default' in description:
        default = description["default"]
        click.echo(f'\nEnter {description["text"]}. Default = [{click.style(default,fg="green")}]')
        answer = get_answer()
        if not answer:
            answer = default

    elif 'choices' in description:
        choices = description["choices"]
        click.echo(f'\nEnter {description["text"]} [{click.style(choices[0],fg="blue")}]\nChoose between:')
        for i,choice in enumerate(choices):
            if i ==0 :
                click.echo(f'  {i}: {click.style(choice, fg="green")} (default)')
            else:
                click.echo(f'  {i}: {click.style(choice, fg="blue")}')

        while 1:
            answer = get_answer()
            if not answer:
                answer = choices[0]
                break
            try:
                i = int(answer)
                answer = choices[i]
                break
            except ValueError:
                result = []
                for choice in choices:
                    if choice.startswith(answer):
                        result.append(choice)
                if len(result) == 1:
                    answer = result[0]
                    break
                    
        print(f'>: {answer}')

    else:
        click.echo(f'\nEnter {description["text"]}')

        while 1:
            answer = get_answer()
            if answer:
                break

    pp = description.get('postprocess', None)
    if pp:
        answer = pp(answer)

    return answer

# eof



# -*- coding: utf-8 -*-
"""Command line interface {{tmpl.app_name}} (no sub-commands)."""

import sys

import click


@click.command()
@click.option('-v', '--verbosity', count=True
             , help="The verbosity of the program."
             , default=1
             )
def main(verbosity):
    """Command line interface {{tmpl.app_name}}.
    
    A 'hello' world CLI example.
    """
    
    for _ in range(verbosity):
        click.echo("running {{tmpl.app_name}}")

if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover
#eodf



# -*- coding: utf-8 -*-
"""
Command line interface {{tmpl.app_name}}.
"""

import sys
from types import SimpleNamespace

import click


@click.group()
@click.option('-v', '--verbosity', count=True
             , help="The verbosity of the program output."
             , default=1
             )
@click.pass_context
def main(ctx, verbosity):
    """Command line interface {{tmpl.app_name}}.
    
    A 'hello' world CLI example.
    """
    # store global options in ctx.obj
    ctx.obj = SimpleNamespace(verbosity=verbosity)

    click.echo(f"running {{tmpl.app_name}}")


@main.command()
@click.argument('who', default='world')
@click.option('-u', '--uppercase'
             , help="Print 'Hello world' in uppercase."
             , default=False, is_flag=True
             )
@click.pass_context
def hello(ctx, who, uppercase):
    """Subcommand hello.
    
    :param str who: whom to say hello to.
    """
    msg = "Hello " + who
    if uppercase:
        msg = msg.upper()
    for i in range(ctx.obj.verbosity):
        print(i,msg)

    return 0

if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover
#eodf



# -*- coding: utf-8 -*-

"""
Module {{tmpl.import_lib}}
==========================

A submodule for ...

"""

# your imports here ...

def greet(to=''):
	"""Say "Hello <to>!".
	
	:param str to: whom you want to say hello to.
	"""
	return f"Hello {to}!"


# -*- coding: utf-8 -*-

"""Tests for {{tmpl.package_name}} package."""

import {{tmpl.package_name}}


def test_hello_noargs():
    """Test for {{tmpl.package_name}}.hello()."""
    s = {{tmpl.package_name}}.hello()
    assert s == "Hello world"


def test_hello_me():
    """Test for {{tmpl.package_name}}.hello('me')."""
    s = {{tmpl.package_name}}.hello('me')
    assert s == "Hello me"


# ==============================================================================
# The code below is for debugging a particular test in eclipse/pydev.
# (otherwise all tests are normally run with pytest)
# Make sure that you run this code with the project directory as CWD, and
# that the source directory is on the path
# ==============================================================================
if __name__ == "__main__":
    the_test_you_want_to_debug = test_hello_noargs

    print("__main__ running", the_test_you_want_to_debug)
    the_test_you_want_to_debug()
    print('-*# finished #*-')

# eof


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `{{tmpl.project_name}}.{{tmpl.cli_app_name}} ` CLI."""

import pytest

from click.testing import CliRunner
from click import echo

from click.testing import CliRunner

from {{tmpl.package_name}}.{{tmpl.cli_app_name}} import main


def test_hello():
    runner = CliRunner()
    result = runner.invoke(main, ['-vv','hello'])
    print(result.output)
    assert 'Hello world' in result.output


# ==============================================================================
# The code below is for debugging a particular test in eclipse/pydev.
# (normally all tests are run with pytest)
# ==============================================================================
if __name__ == "__main__":
    the_test_you_want_to_debug = test_hello

    print(f"__main__ running {the_test_you_want_to_debug}")
    the_test_you_want_to_debug()
    print('-*# finished #*-')
# eof



# -*- coding: utf-8 -*-
"""
Module et_micc2.expand
======================

Helper functions for dealing with *Cookiecutter* templates.
"""

import os, shutil, platform
from pathlib import Path
import json

import click
# from cookiecutter.main import cookiecutter

import et_micc2.logger

EXIT_OVERWRITE = -3
__FILE__ = Path(__file__).resolve() 
import et_micc2.tmpl


def expand_templates(options):
    for template in options.templates:
        path_to_template = Path(__FILE__).parent / 'templates' / template
        if not path_to_template.exists():
            raise RuntimeError(f'Template not found: {path_to_template}')

        if template.startswith('submodule-'):
            if template.endswith('-test'):
                destination = options.project_path / 'tests' / options.package_name / options.module_location_relative
            else:
                destination = options.project_path / options.package_name / options.module_location_relative
        else: # not a sub-module, either top-level package or cli
            destination = options.project_path.parent
        try:
            et_micc2.tmpl.expand_folder(path_to_template, destination, options.template_parameters.data)
        except ValueError as exc:
            return exc.args[0]


def resolve_template(template):
    """Compose the absolute path of a template."""
    
    if  template.startswith('~') or template.startswith(os.sep):
        pass # absolute path
    elif os.sep in template:
        # reative path
        template = Path.cwd() / template
    else:
        # just the template name
        template = Path(__FILE__).parent / 'templates' / template
      
    if not template.exists():
        print(f'template=={template}')
        raise AssertionError(f"Inexisting template {template}")
    
    return template


def set_preferences(micc_file):
    """Set the preferences in *micc_file*.
    
    (This function requires user interaction!)
    
    :param Path micc_file: path to a json file.
    """
    with micc_file.open() as f:
        preferences = json.load(f)

    ty = None
    for parameter,description in preferences.items():
        if not description['default'].startswith('{{ '):
            if 'type' in description:
                ty = description['type']
                description['type'] = eval(ty)
            answer = click.prompt(**description)
            if ty:
                # set the string back
                description['type'] = ty
            preferences[parameter]['default'] = answer
            
    with micc_file.open(mode='w') as f:
        json.dump(preferences, f, indent=2)
        
    return preferences


def get_preferences(micc_file):
    """Get the preferences from *micc_file*.
    
    (This function requires user interaction if no *micc_file* was provided!)

    :param Path micc_file: path to a json file.
    """
    if micc_file.samefile('.'):
        # There is no et_micc2 file with preferences yet.
        dotmicc = Path().home() / '.et_micc2'
        dotmicc.mkdir(exist_ok=True)
        dotmicc_miccfile = dotmicc / 'micc2.json'
        if dotmicc_miccfile.exists():
            preferences = get_preferences(dotmicc_miccfile)
        else:
            preferences = None
            # micc_file_template = Path(__file__).parent / 'micc2.json'
            # shutil.copyfile(str(micc_file_template),str(dotmicc_miccfile))
            # preferences = set_preferences(dotmicc_miccfile)
    else:
        with micc_file.open() as f:
            preferences = json.load(f)

    return preferences


def get_template_parameters(preferences):
    """Get the template parameters from the preferences.
    
    :param dict|Path preferenes:
    :returns: dict of (parameter name,parameter value) pairs.
    """
    if isinstance(preferences,dict):
        template_parameters = {}
        for parameter,description in preferences.items():
            template_parameters[parameter] = description['default']
    elif isinstance(preferences,Path):
        with preferences.open() as f:
            template_parameters = json.load(f)
    else:
        raise RuntimeError()
    
    return template_parameters
    
    
# def expand_templates(options):
#     """Expand a list of cookiecutter :py:obj:`templates` in directory :py:obj:`project_path`.
#
#     Expanding templates may require overwriting pre-existing files. *Micc* handles this
#     situation in different ways:
#
#     * If :py:obj:`options.overwrite` equals :py:const:`False` the exansion will
#       fail without overwriting any pre-existing files. The project is not modified. A
#       warning is produced. This is the default. To continue, rerun the command with one
#       of the two options below.
#     * If :py:obj:`options.overwrite` equals :py:const:`True` the exansion will
#       overwrite any pre-existing files without backup, and produce a warning, listing
#       the overwritten files.
#     * If :py:obj:`options.backup` equals :py:const:`True` pre-existing files
#       will be backed up (.bak) before the new files are expanded. If anything went
#       wrong, you can inspect the backup files, and correct the errors manually.
#
#     :param types.SimpleNamespace options: namespace object with
#         options accepted by et_micc2 commands. Relevant attributes are
#
#         * templates: ordered list of (paths to) cookiecutter templates that
#           will be expanded as they appear. The template parameters are propagated
#           from each template to the next.
#         * **verbosity**
#         * **project_path**: Path to the project on which the command operates.
#         * **template_parameters**: extra template parameters not read from *micc_file*
#     """
#     templates = options.templates
#     if not isinstance(templates, list):
#         templates = [templates]
#     project_path = options.project_path
#     project_path.mkdir(parents=True, exist_ok=True)
#     output_dir = project_path.parent
#     micc_logger = options.logger
#
#     # list existing files that would be overwritten if options.overwrite==True
#     existing_files = {}
#     for template in templates:
#         template = resolve_template(template)
#         # write a cookiecutter.json file in the cookiecutter template directory
#         cookiecutter_json = template / 'cookiecutter.json'
#         options.template_parameters.save(cookiecutter_json)
#
#         # run cookiecutter in an empty temporary directory to check if there are any
#         # existing project files that would be overwritten.
#         tmp = output_dir / '_cookiecutter_tmp_'
#         if tmp.exists():
#             shutil.rmtree(tmp)
#         tmp.mkdir(parents=True, exist_ok=True)
#
#         # expand the Cookiecutter template in a temporary directory,
#         cookiecutter( str(template)
#                     , no_input=True
#                     , overwrite_if_exists=True
#                     , output_dir=str(tmp)
#                     )
#
#         # find out if there are any files that would be overwritten.
#         os_name = platform.system()
#         for root, _, files in os.walk(tmp):
#             if root==tmp:
#                 continue
#             else:
#                 root2 = os.path.relpath(root,tmp)
#             for f in files:
#                 if os_name=="Darwin" and f==".DS_Store":
#                     continue
#                 file = output_dir / root2 / f
#                 if file.exists():
#                     if not template in existing_files:
#                         existing_files[template] = []
#                     existing_files[template].append(file)
#
#         if existing_files:
#             if options.backup:
#                 micc_logger.warning("Pre-existing files that will be backed up ('--backup' specified):\n")
#                 micc_logger.indent(2)
#                 for files in existing_files.values():
#                     for src in files:
#                         src = str(src)
#                         dst = src + '.bak'
#                         shutil.copyfile(src, dst)
#                         micc_logger.warning(f"{src} -> {dst}")
#                 micc_logger.dedent()
#
#             elif not options.overwrite:
#                 micc_logger.warning("Pre-existing files that would be overwritten:\n")
#                 micc_logger.indent(2)
#                 for files in existing_files.values():
#                     for src in files:
#                         micc_logger.warning(str(src))
#                 micc_logger.dedent()
#                 click.secho("Aborting because 'overwrite==False'.\n"
#                             "  Rerun the command with the '--backup' flag to first backup these files (*.bak).\n"
#                             "  Rerun the command with the '--overwrite' flag to overwrite these files without backup.\n"
#                             "Aborting."
#                            , fg='bright_red'
#                            )
#                 return EXIT_OVERWRITE
#             else:
#                 micc_logger.warning(f"'--overwrite' specified: pre-existing files will be overwritten WITHOUT backup:\n")
#                 for files in existing_files.values():
#                     for src in files:
#                         micc_logger.warning(f"     overwriting {src}")
#
#     # Now we can safely overwrite pre-existing files.
#     micc_logger.debug(f"Expanding templates using these parameters:\n{options.template_parameters}")
#     for template in templates:
#         template = resolve_template(template)
#         micc_logger.debug(f"Expanding template {template}.")
#         cookiecutter( str(template)
#                     , no_input=True
#                     , overwrite_if_exists=True
#                     , output_dir=str(output_dir)
#                     )
#         # Clean up (see issue #7)
#         cookiecutter_json = template / 'cookiecutter.json'
#         cookiecutter_json.unlink()
#
#     # Clean up
#     if tmp.exists():
#         shutil.rmtree(str(tmp))
#
#     return 0


#eof


# -*- coding: utf-8 -*-

"""
Application micc2
"""
import subprocess



def sys_path_helper():
    """Make sure that et_micc2 can be imported in case this file is executed as::

            (.venv)> python et_micc2/cli_micc.py <args>
    """
    try:
        import et_micc2
    except ModuleNotFoundError:
        p = Path(__file__) / '..' / '..'
        sys.path.insert(0, str(p.resolve()))


import os, sys, shutil
from types import SimpleNamespace
from pathlib import Path

import click

sys_path_helper()
from et_micc2.project import Project, micc_version, error
import et_micc2.logger
import et_micc2.config
import pkg_resources

if '3.8' < sys.version:
    from et_micc2.check_environment import check_cmd

__template_help = "Ordered list of Cookiecutter templates, or a single Cookiecutter template."


def underscore2space(text):
    return text.replace('_', ' ')

_subcmds_supporting_overwrite_preferences = ('setup', 'create')
_cfg_filename = 'micc3.cfg'
_cfg_dir = Path.home() / '.micc2'

_preferences_setup = { "full_name":
                           { "text": "your full name"
                           , "postprocess": underscore2space }
                     , "email":
                           { "text": "your e-mail address" }
                     , "github_username":
                           { "default": ""
                           , "text": "your github username (leave empty if you do not have one,\n"
                                     "  or create one first at https://github.com/join)" }
                     , "sphinx_html_theme":
                           { "default": "sphinx_rtd_theme"
                           , "text": "Html theme for sphinx documentation" }
                     # , "software_license":
                     #       { "choices": [ 'MIT license', 'GNU General Public License v3'
                     #                    , 'BSD license', 'ISC license'
                     #                    , 'Apache Software License 2.0', 'Not open source']
                     #       , "text": "the default software license" }
                     }


####################################################################################################
# main
####################################################################################################
@click.group()
@click.option('-v', '--verbosity', count=True
    , help="The verbosity of the program output."
    , default=1
)
@click.option('-p', '--project-path'
    , help="The path to the project directory. "
           "The default is the current working directory."
    , default='.'
    , type=str
)
@click.option('--clear-log'
    , help="If specified clears the project's ``et_micc2.log`` file."
    , default=False, is_flag=True
)
# optionally overwrite preferences (supporting sub-commands only):
@click.option('--full-name'
    , help=f"Overwrite preference `full_name`, use underscores for spaces. (supporting sub-commands only {_subcmds_supporting_overwrite_preferences})"
    , default=''
)
@click.option('--email'
    , help=f"Overwrite preference `email`. (supporting sub-commands only {_subcmds_supporting_overwrite_preferences})"
    , default=''
)
@click.option('--github-username'
    , help=f"Overwrite preference `github_username`. (supporting sub-commands only {_subcmds_supporting_overwrite_preferences})"
    , default=''
)
@click.option('--sphinx-html-theme'
    , help=f"Overwrite preference `sphinx_html_theme`. (supporting sub-commands only {_subcmds_supporting_overwrite_preferences})"
    , default=''
)
# @click.option('--software-license'
#     , help=f"Overwrite preference `software_license`. (supporting sub-commands only {_subcmds_supporting_overwrite_preferences})"
#     , default=''
# )
@click.option('--git-default-branch'
    , help=f"Overwrite preference `git_default_branch`. (supporting sub-commands only {_subcmds_supporting_overwrite_preferences})"
    , default=''
)
@click.option('--minimal_python_version'
    , help=f"Overwrite preference `minimal_python_version`. (supporting sub-commands only {_subcmds_supporting_overwrite_preferences})"
    , default=''
)
# end of preferences overwrite options
# Don't put any options below, otherwise they will be treated as overwrite preferences.
@click.version_option(version=micc_version())
@click.pass_context
def main( ctx, verbosity, project_path, clear_log
        , **overwrite_preferences
        ):
    """Micc2 command line interface.

    All commands that change the state of the project produce some output that
    is send to the console (taking verbosity into account). It is also sent to
    a logfile ``et_micc2.log`` in the project directory. All output is always appended
    to the logfile. If you think the file has gotten too big, or you are no more
    interested in the history of your project, you can specify the ``--clear-log``
    flag to clear the logfile before any command is executed. In this way the
    command you execute is logged to an empty logfile.

    See below for (sub)commands.
    """
    if verbosity > 1:
        print(f"micc2 ({et_micc2.__version__}) using Python", sys.version.replace('\n', ' '), end='\n\n')

    if clear_log:
        os.remove(project_path / 'micc.log')

    ctx.obj = SimpleNamespace(
        verbosity=verbosity,
        project_path=Path(project_path).resolve(),
        default_project_path=(project_path=='.'),
        clear_log=clear_log,
        _cfg_filename=_cfg_filename,
        _cfg_dir=_cfg_dir,
        invoked_subcommand=ctx.invoked_subcommand

    )

    overwrite_preferences_set = {}
    if ctx.invoked_subcommand in _subcmds_supporting_overwrite_preferences:
        # Remove overwrite_preferences which have not been explicitly set:
        for key,value in overwrite_preferences.items():
            if value:
                if key == 'full_name':
                    value = underscore2space(value)
                # elif key == 'software_license':
                #     for lic in _preferences_setup[key]['choices']:
                #         if lic.startswith(value):
                #             value = lic

                overwrite_preferences_set[key] = value
    else:
        for key, value in overwrite_preferences.items():
            if value:
                print(f'Warning: overwriting preferences is supported only for subcommands `setup` and `create`.\n'
                      f'         Ignoring `{key}={value}`.')

    try:
        preferences = et_micc2.config.Config(file_loc=ctx.obj.project_path / _cfg_filename)
    except FileNotFoundError:
        try:
            preferences = et_micc2.config.Config(file_loc=_cfg_dir / _cfg_filename)
        except FileNotFoundError:
            preferences = None

    if preferences is None:
        # no preferences file found
        if ctx.invoked_subcommand == 'setup':
            # we're about to create one.
            pass
        else:
            # other commands cannot be executed without a preferences file:
            print(f"ERROR: No configuration file found in \n"
                  f"       - {ctx.obj.project_path / 'micc2.cfg'}\n"
                  f"       - {Path.home() / '.et-micc2/.et-micc2.cfg'}\n"
                  f"Run `micc2 setup` first.")
            ctx.exit(1)

    # pass on the preferences and their overwrites to the subcommands (The overwrite_preferences
    # are empty if the invoked subcommands do not support it
    ctx.obj.preferences = preferences
    ctx.obj.overwrite_preferences = overwrite_preferences_set


####################################################################################################
# setup
####################################################################################################
@main.command()
@click.option('--force', '-f', is_flag=True
    , help="Overwrite existing setup."
    , default=False
)
@click.option('--modify', '-m', is_flag=True
    , help="Modify existing setup."
    , default=False
)
@click.pass_context
def setup( ctx
         , force
         , modify
        ):
    """Setup your micc preferences.

    This command must be run once before you can use micc to manage your projects.
    """
    context = ctx.obj

    if context.preferences is None:
        pass
    else:
        if modify:
            force = True
        if force:
            click.secho(f"Overwriting earlier setup: \n    {context.preferences['file_loc']}", fg='bright_red')
            click.secho("Enter a suffix for the configuration directory if you want to make a backup.")
            while 1:
                suffix = input(':> ')

                if suffix:  # make backup
                    p_cfg_dir = Path(context.preferences['file_loc']).parent
                    p_cfg_dir_renamed = p_cfg_dir.parent / (p_cfg_dir.name + suffix)
                    if p_cfg_dir_renamed.exists():
                        click.secho(f'Error: suffix `{suffix}` is already in use, choose another.', fg='bright_red')
                    else:
                        p_cfg_dir.rename(p_cfg_dir_renamed)
                        break
                else:
                    break

            # forget the previous preferences.
            if not modify:
                context.preferences = None
        else:
            print(f"Micc2 has already been set up:\n"
                  f"    {context.preferences['file_loc']}\n"
                  f"Use '--force' or '-f' to overwrite the existing preferences file.")
            ctx.exit(1)

    selected = {}
    for name, description in _preferences_setup.items():
        if name in context.overwrite_preferences:
            selected[name] = context.overwrite_preferences[name]
        else:
            if modify: # use the previous setting as default
                if not 'choices' in description:
                    description['default'] = context.preferences[name]
            try:
                selected[name] = et_micc2.config.get_param(name, description)
            except KeyboardInterrupt:
                print('Interupted - Preferences are not saved.')
                ctx.exit(1)

    # set some preferences for which the default is almost always ok
    selected['version'] = '0.0.0'  # default initial version number of a new projec
    selected["github_repo"] = "{{cookiecutter.project_name}}"  # default github repo name for a project
    selected["git_default_branch"] = "master" # default git branch
    selected["minimal_python_version"] = "3.7"  # default minimal Python version"
    selected["py"] = "py"

    # Transfer the selected preferences to a Config object and save it to disk.
    context.preferences = et_micc2.config.Config(**selected)
    save_to = _cfg_dir / _cfg_filename
    print(f'These preferences are saved to {save_to}:\n{context.preferences}')
    answer = input("Continue? yes/no >:")
    if not answer.lower().startswith('n'):
        context.preferences.save(save_to, mkdir=True)
        print(f'Preferences saved to {save_to}.')
        print('Configuring git:')
        if shutil.which('git'):
            cmd = ['git','config', '--global', 'user.name', context.preferences['github_username']]
            print(f'  {" ".join(cmd)}')
            subprocess.run(cmd)
            cmd = ['git','config', '--global', 'user.email', context.preferences['email']]
            print(f'  {" ".join(cmd)}')
            subprocess.run(cmd)
            # git personal access token
            print('Paste your GitHub personal access token, or the file location containing it:')
            pat = input(':> ')
            if pat:
                p = Path(pat)
                dst = _cfg_dir/f'{context.preferences["github_username"]}.pat'
                if p.exists():
                    shutil.copyfile(p, dst)
                else:
                    with dst.open(mode='w+') as f:
                        f.write(pat)
            else:
                print('No GitHub personal access token specified.')

        else:
            print('WARNING: git not found, could not configure git.')
            ctx.exit(1)

    else:
        print('Interrupted. Preferences not saved.')
        ctx.exit(1)

    # make the scripts directory available through a symlink in the configuration directory:
    if sys.platform=='win32':
        if hasattr(os, 'enable_symlink') and not os.enable_symlink():
            raise WindowsError('Symlinking is disabled.')
    src = Path(pkg_resources.get_distribution('et-micc2').location) / 'et_micc2/scripts'
    dst = _cfg_dir / 'scripts'
    dst.unlink(missing_ok=True)
    os.symlink( src=src, dst=dst
              , target_is_directory=True    # needed on windows
              )

    ctx.exit(0)

####################################################################################################
# create
####################################################################################################
@main.command()
@click.option('--publish'
    , help="If specified, verifies that the package name is available on PyPI.\n"
           "If the result is False or inconclusive the project is NOT created."
    , default=False, is_flag=True
)
# From now on we only support package structure - simplifies the code significantly
# @click.option('-p', '--package'
#     , help="Create a Python project with a package structure rather than a module structure:\n\n"
#            "* package structure = ``<module_name>/__init__.py``\n"
#            "* module  structure = ``<module_name>.py`` \n"
#     , default=False, is_flag=True
# )
@click.option('-d', '--description'
    , help="Short description of your project."
    , default='<Enter a one-sentence description of this project here.>'
)
# suppressed
# @click.option('-T', '--template', help=__template_help, default=[])
@click.option('-n', '--allow-nesting'
    , help="If specified allows to nest a project inside another project."
    , default=False, is_flag=True
)
@click.option('--module-name'
    , help="use this name for the module, rather than deriving it from the project name."
    , default=''
)
@click.option('--no-git'
    , help="Create the project without git support. "
    , default=False, is_flag=True
)
@click.option('--remote'
    , help="Create remote repo on github. Choose from 'public'(=default), 'private', or 'none'."
    , default='public'
)
@click.argument('name', type=str, default='')
@click.pass_context
def create( ctx
          , name
          # , package
          , module_name
          , description
          # , template
          , allow_nesting
          , publish
          , no_git
          , remote
          ):
    """Create a new project skeleton.

    The project name is taken to be the last directory of the *project_path*.
    If this directory does not yet exist, it is created. If it does exist already, it
    must be empty.

    The package name is the derived from the project name, taking the
    `PEP8 module naming rules <https://www.python.org/dev/peps/pep-0008/#package-and-module-names>`_
    into account:

    * all lowercase.
    * dashes ``'-'`` and spaces ``' '`` replaced with underscores ``'_'``.
    * in case the project name has a leading number, an underscore is prepended ``'_'``.

    If *project_path* is a subdirectory of a micc project, *micc* refuses to continu,
    unless ``--allow-nesting`` is soecified.
    """
    context = ctx.obj

    if name:
        if not context.default_project_path:
            # global option -p and argument name were both specified.
            print( "ERROR: you specified both global option -p and argument 'name':"
                  f"         -p -> {context.project_path}"
                  f"         name -> {name}"
                   "       You must choose one or the other, not both."
                 )
            ctx.exit(-1)
        else:
            # overwrite the -p global option so the project will be created:
            context.project_path = Path(name).resolve()
            context.default_project_path = False

    # context.package_structure = package
    context.publish = publish
    context.module_name = module_name
    if not remote in ['public','private', 'none']:
        print(f"ERROR: --remote={remote} is not recognized. Valid options are:\n"
              f"       --remote=public\n"
              f"       --remote=private\n"
              f"       --remote=none\n"
              )
        ctx.exit(-1)

    context.no_git = no_git
    if no_git:
        context.remote = 'none'
    else:
        context.remote = None if remote=='none' else remote

    context.templates = ['top-level-package']

    context.allow_nesting = allow_nesting

    context.preferences.update(context.overwrite_preferences)

    context.template_parameters = { 'project_short_description': underscore2space(description) }

    try:
        project = Project(context)
        project.create_cmd()
    except RuntimeError:
        ctx.exit(2)


####################################################################################################
# convert_to_package
####################################################################################################
@main.command()
@click.option('--overwrite', '-o'
    , help="Overwrite pre-existing files (without backup)."
    , is_flag=True, default=False
)
@click.option('--backup', '-b'
    , help="Make backup files (.bak) before overwriting any pre-existing files."
    , is_flag=True, default=False
)
@click.pass_context
def convert_to_package(ctx, overwrite, backup):
    """Convert a Python module project to a package.

    A Python *module* project has only a ``<package_name>.py`` file, whereas
    a Python *package* project has ``<package_name>/__init__.py`` and can contain
    submodules, such as Python modules, packages and applications, as well as
    binary extension modules.

    This command also expands the ``package-general-docs`` template in this
    project, which adds a ``AUTHORS.rst``, ``HISTORY.rst`` and ``installation.rst``
    to the documentation structure.
    """
    context = ctx.obj
    context.overwrite = overwrite
    context.backup = backup

    try:
        project = Project(context)
        with et_micc2.logger.logtime(context):
            project.module_to_package_cmd()
    except RuntimeError:
        if project.exit_code == et_micc2.expand.EXIT_OVERWRITE:
            context.logger.warning(
                f"It is normally ok to overwrite 'index.rst' as you are not supposed\n"
                f"to edit the '.rst' files in '{context.project_path}{os.sep}docs.'\n"
                f"If in doubt: rerun the command with the '--backup' flag,\n"
                f"  otherwise: rerun the command with the '--overwrite' flag,\n"
            )
        ctx.exit(project.exit_code)

####################################################################################################
# info
####################################################################################################
@main.command()
@click.option('--name', is_flag=True
    , help="print the project name."
    , default=False
)
@click.option('--version', is_flag=True
    , help="print the project version."
    , default=False
)
@click.pass_context
def info(ctx,name,version):
    """Show project info.

    * file location
    * name
    * version number
    * structure (with ``-v``)
    * contents (with ``-vv``)

    Use verbosity to produce more detailed info.
    """
    context = ctx.obj

    try:
        project = Project(context)
    except RuntimeError:
        ctx.exit(-2)

    if name:
        print(project.package_name)
        return
    if version:
        print(project.version)
        return
    else:
        with et_micc2.logger.logtime(project):
            try:
                project.info_cmd()
            except RuntimeError:
                ctx.exit(project.exit_code)


####################################################################################################
# version
####################################################################################################
@main.command()
@click.option('-M', '--major'
    , help='Increment the major version number component and set minor and patch components to 0.'
    , default=False, is_flag=True
)
@click.option('-m', '--minor'
    , help='Increment the minor version number component and set minor and patch component to 0.'
    , default=False, is_flag=True
)
@click.option('-p', '--patch'
    , help='Increment the patch version number component.'
    , default=False, is_flag=True
)
@click.option('-r', '--rule'
    , help='Any semver 2.0 version string.'
    , default=''
)
@click.option('-t', '--tag'
    , help='Create a git tag for the new version, and push it to the remote repo.'
    , default=False, is_flag=True
)
@click.option('-s', '--short'
    , help='Print the version on stdout.'
    , default=False, is_flag=True
)
@click.option('-d', '--dry-run'
    , help='bumpversion --dry-run.'
    , default=False, is_flag=True
)
@click.pass_context
def version(ctx, major, minor, patch, rule, tag, short, dry_run):
    """Modify or show the project's version number."""
    context = ctx.obj

    if rule and (major or minor or patch):
        msg = ("Both --rule and --major|--minor|--patc specified.")
        click.secho("[ERROR]\n" + msg, fg='bright_red')
        ctx.exit(1)
    elif major:
        rule = 'major'
    elif minor:
        rule = 'minor'
    elif patch:
        rule = 'patch'

    context.rule = rule
    context.short = short
    context.dry_run = dry_run

    try:
        project = Project(context)
    except RuntimeError:
        ctx.exit(2)

    with et_micc2.logger.logtime(project):
        try:
            project.version_cmd()
        except RuntimeError:
            ctx.exit(2)
        else:
            if tag:
                project.tag_cmd()


####################################################################################################
# tag
####################################################################################################
@main.command()
@click.pass_context
def tag(ctx):
    """Create a git tag for the current version and push it to the remote repo."""
    context = ctx.obj

    try:
        project = Project(context)
        project.tag_cmd()
    except RuntimeError:
        ctx.exit(2)


####################################################################################################
# add
####################################################################################################
@main.command()
@click.option('--cli'
    , default=False, is_flag=True
    , help="Add a CLI with a single command."
)
@click.option('--clisub'
    , default=False, is_flag=True
    , help="Add a CLI with a group of sub-commands rather than a single command CLI."
)
@click.option('--py'
    , default=False, is_flag=True
    , help="Add a Python module (package structure)."
)
# in micc3 we only allow module/__init__.py structure
# @click.option('--package'
#     , help="Add a Python module with a package structure rather than a module structure:\n\n"
#            "* module  structure = ``<module_name>.py`` \n"
#            "* package structure = ``<module_name>/__init__.py``\n\n"
#            "Default = module structure."
#     , default=False, is_flag=True
# )
@click.option('--f90'
    , default=False, is_flag=True
    , help="Add a f90 binary extionsion module (Fortran)."
)
@click.option('--cpp'
    , default=False, is_flag=True
    , help="Add a cpp binary extionsion module (C++)."
)
# @click.option('-T', '--templates', default='', help=__template_help)
@click.option('--overwrite', is_flag=True
    , help="Overwrite pre-existing files (without backup)."
    , default=False
)
@click.option('--backup', is_flag=True
    , help="Make backup files (.bak) before overwriting any pre-existing files."
    , default=False
)
@click.argument('name', type=str)
@click.pass_context
def add(ctx
        , name
        , cli, clisub
        , py
        # , package
        , f90
        , cpp
        # , templates
        , overwrite
        , backup
        ):
    """Add a component to the projcect, e.g. CLI, sub-module, sub-package, binary extension 
    module (C++, Fortran, C++/CUDA )

    :param str name: name of the component. Maybe path-like relative to package directory to
        create sub-sub-modules.
    """
    context = ctx.obj
    context.add_name = name
    
    context.flag_cli = cli
    context.flag_clisub = clisub
    context.flag_py = py
    # context.package = package
    context.flag_f90 = f90
    context.flag_cpp = cpp
        
    # context.templates = templates
    context.template_parameters = context.preferences.data

    # TODO: remove overwrite and backup?
    context.overwrite = overwrite
    context.backup = backup

    try:
        project = Project(context)
        n_selected = cli+clisub+py+f90+cpp # yes, you can add bool variables, they are literally 0|1
        if n_selected == 0:
            error('You must select a component type:\n    add flag: --py | --cli | --clisub | --f90 | --cpp.')
        if n_selected > 1:
            error(f'You must select just one component type, not {n_selected}.')
        with et_micc2.logger.logtime(project):
            project.add_cmd()
    except RuntimeError as exc:
        ctx.exit(exc.args[1])


####################################################################################################
# mv
####################################################################################################
@main.command()
@click.option('--silent', is_flag=True
    , help="Do not ask for confirmation on deleting a component."
    , default=False
)
@click.option('--entire-package', is_flag=True
    , help="Replace all occurences of <cur_name> in the entire package and in the ``tests`` directory."
    , default=False
)
@click.option('--entire-project', is_flag=True
    , help="Replace all occurences of <cur_name> in the entire project."
    , default=False
)
@click.argument('cur_name', type=str)
@click.argument('new_name', type=str, default='')
@click.pass_context
def mv(ctx, cur_name, new_name, silent, entire_package, entire_project):
    """Rename or remove a component, i.e an app (CLI) or a submodule.

    :param cur_name: name of component to be removed or renamed.
    :param new_name: new name of the component. If empty, the component will be removed.
    """
    context = ctx.obj

    context.cur_name = cur_name
    context.new_name = new_name
    context.silent = silent
    if new_name:
        context.entire_package, context.entire_project =  entire_package, entire_project
    # else these flags are ignored.

    try:
        project = Project(context)
        with et_micc2.logger.logtime(context):
            project.mv_component()
    except RuntimeError:
        ctx.exit(2)


####################################################################################################
# build
####################################################################################################
@main.command()
@click.argument('module', type=str, default='')
@click.option('-b', '--build-type'
    , help="build type: any of the standard CMake build types: "
           "Release (default), Debug, RelWithDebInfo, MinSizeRel."
    , default=''
)
@click.option('--clean'
    , help="Perform a clean build, removes the build directory before the build, if there is one. "
           "Note that this option is necessary if the extension's ``CMakeLists.txt`` was modified."
    , default=False, is_flag=True
)
@click.option('--cleanup'
    , help="Cleanup remove the build directory after a successful build."
    , default=False, is_flag=True
)
@click.pass_context
def build( ctx
         , module
         , build_type
         , clean
         , cleanup
         ):
    """Build binary extensions.

    :param str module: build a binary extension module. If not specified or all binary
        extension modules are built.
    """
    # Warn for non standard build_type:
    if not build_type in ('','Release', 'Debug', 'RelWithDebInfo', 'MinSizeRel'):
        print(f'[Warning]\nNon-standard build type: `{build_type}`\n'
              f'(Standard build types are: Release, Debug, RelWithDebInfo, MinSizeRel)')
        answer = input('>: Continue? (y/N)')
        if answer != 'y':
            print('')
            ctx.exit(-1)

    context = ctx.obj
    context.build_options = SimpleNamespace( module_to_build = module
                                           , clean           = clean
                                           , cleanup         = cleanup
                                           , cmake           = {}
                                           )
    if build_type:
        context.build_options.cmake['CMAKE_BUILD_TYPE'] = build_type

    try:
        project = Project(context)
        with et_micc2.logger.logtime(context):
            project.build_cmd()
    except RuntimeError:
        ctx.exit(2)


####################################################################################################
# check
####################################################################################################
@main.command()
# @click.option('--force', '-f', is_flag=True
#     , help="Overwrite existing setup."
#     , default=False
# )
@click.pass_context
def check( ctx
         ):
    """Check wether the current environment has all the neccessary tools available.

    Python packages:

    * Numpy
    * Pybind11
    * sphinx
    * sphinx-rtd-theme
    * sphinx-click

    Tools:

    * CMake
    * make
    * poetry
    * git
    * gh
    * compilers
    """
    if '3.8' < sys.version:
        check_cmd(ctx.obj)
    else:
        print("`micc2 check` requires python 3.8 or later.")


####################################################################################################
# doc
####################################################################################################
@main.command()
@click.pass_context
@click.argument('what', type=str, default='html')
def doc(ctx, what):
    """Build documentation.

    :param str what: this argument is passed to the make command.
    """
    context = ctx.obj
    context.what = what
    try:
        project = Project(context)
        with et_micc2.logger.logtime(context):
            project.doc_cmd()

    except RuntimeError:
        pass

    ctx.exit(project.exit_code)


####################################################################################################
# venv
####################################################################################################
# @main.command()
# @click.option('--python'
#     , help="path to the Python executable to be used for the virtual environment. "
#            "Default is the current Python. "
#     , default=''
# )
# @click.option('--system-site-packages'
#     , help="path to the Python executable to be used for the virtual environment. "
#            "Default is the current Python. "
#     , default=False, is_flag=True
# )
# @click.argument('name', default='')
# @click.pass_context
# def venv(ctx, name, python, system_site_packages):
#     """Construct a virtual environment for this project. The project is installed in
#     development mode.
#
#     :param str name: name of the virtual environment. default = f'.venv-{project_name}'.
#     """
#     context = ctx.obj
#     if not name:
#         name = f'.venv-{context.project_path.name}'
#     context.venv_name = name
#
#     if not python:
#         python = sys.executable
#     context.python_executable = python
#
#     context.system_site_packages = system_site_packages
#
#     try:
#         project = Project(context)
#         with et_micc2.logger.logtime(context):
#             project.venv_cmd()
#
#     except RuntimeError:
#         pass
#
#     ctx.exit(project.exit_code)


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover

# eof



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for sub-module {{tmpl.import_lib}}."""

import pytest
import {{tmpl.import_lib}}


def test_greet():
    expected = "Hello John!"
    greeting = {{tmpl.package_name}}.{{tmpl.module_name}}.greet("John")
    assert greeting==expected


# ==============================================================================
# The code below is for debugging a particular test in eclipse/pydev.
# (normally all tests are run with pytest)
# ==============================================================================
if __name__ == "__main__":
    the_test_you_want_to_debug = test_greet

    print("__main__ running", the_test_you_want_to_debug)
    the_test_you_want_to_debug()
    print("-*# finished #*-")
# ==============================================================================
